/* 
   HTTP utility functions
   Copyright (C) 1999, Joe Orton <joe@orton.demon.co.uk>
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
   $Id: http_utils.c,v 1.3 1999/12/27 21:45:24 joe Exp $
*/

#include <config.h>

#include <sys/types.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "dates.h"

#include "http_utils.h"

/* HTTP-date parser */
time_t http_dateparse( const char *date ) {
    time_t tmp;
    tmp = rfc1123_parse( date );
    if( tmp == -1 ) {
        tmp = rfc1036_parse( date );
	if( tmp == -1 )
	    tmp = asctime_parse( date );
    }
    return tmp;
}

struct http_buffer *http_buffer_create( void ) {
    struct http_buffer *buf;
    buf = malloc( sizeof(struct http_buffer) );
    if( buf == NULL ) return NULL;
    buf->data = malloc( 512 );
    memset( buf->data, 0, 512 );
    buf->length = 512;
    buf->used = 0;
    return buf;
}

void http_buffer_destroy( struct http_buffer *buf ) {
    if( buf->data ) {
	free( buf->data );
	buf->data = NULL;
    }
    free( buf );
}

int http_buffer_concat( struct http_buffer *buf, ... ) {
    va_list ap;
    char *next;
    size_t totallen = 1; /* initialized to 1 for the terminating \0 */    

    /* Find out how much space we need for all the args */
    va_start( ap, buf );
    do {
	next = va_arg( ap, char * );
	if( next != NULL ) {
	    totallen += strlen( next );
	}
    } while( next != NULL );
    va_end( ap );
    
    /* Grow the buffer */
    if( http_buffer_grow( buf, buf->used + totallen ) )
	return -1;
    
    /* Now append the arguments to the buffer */
    va_start( ap, buf );
    do {
	next = va_arg( ap, char * );
	if( next != NULL ) {
	    strcat( buf->data, next );
	}
    } while( next != NULL );
    va_end( ap );
    
    buf->used += totallen;
    return 0;
}


/* Append zero-terminated string... returns 0 on success or -1 on
 * realloc failure. */
int http_buffer_zappend( struct http_buffer *buf, const char *str ) {
    size_t len = strlen( str );
    if( http_buffer_grow( buf, buf->used + len + 1 ) ) {
	return -1;
    }
    strcat( buf->data, str );
    buf->used += len;
    return 0;
}

int http_buffer_append( struct http_buffer *buf, const char *data, size_t len ) {
    if( http_buffer_grow( buf, buf->used + len ) ) {
	return -1;
    }
    memcpy( buf->data+buf->used, data, len );
    buf->used += len;
    return 0;
}
